Index: channel.c
===================================================================
--- channel.c	(revision 69732)
+++ channel.c	(working copy)
@@ -1047,21 +1047,19 @@
 /* Detach a spy from it's channel */
 static void spy_detach(struct ast_channel_spy *spy, struct ast_channel *chan)
 {
-	ast_mutex_lock(&spy->lock);
-
 	/* We only need to poke them if they aren't already done */
 	if (spy->status != CHANSPY_DONE) {
+		ast_mutex_lock(&spy->lock);
 		spy->status = CHANSPY_STOP;
 		spy->chan = NULL;
 		if (ast_test_flag(spy, CHANSPY_TRIGGER_MODE) != CHANSPY_TRIGGER_NONE)
 			ast_cond_signal(&spy->trigger);
+		ast_mutex_unlock(&spy->lock);
 	}
 	
 	/* Print it out while we still have a lock so the structure can't go away (if signalled above) */
 	ast_log(LOG_DEBUG, "Spy %s removed from channel %s\n", spy->type, chan->name);
 
-	ast_mutex_unlock(&spy->lock);
-
 	return;
 }
 
@@ -1073,13 +1071,10 @@
 		return;
 
 	AST_LIST_TRAVERSE_SAFE_BEGIN(&chan->spies->list, spy, list) {
-		ast_mutex_lock(&spy->lock);
 		if ((spy->type == type) && (spy->status == CHANSPY_RUNNING)) {
-			ast_mutex_unlock(&spy->lock);
 			AST_LIST_REMOVE_CURRENT(&chan->spies->list, list);
 			spy_detach(spy, chan);
-		} else
-			ast_mutex_unlock(&spy->lock);
+		}
 	}
 	AST_LIST_TRAVERSE_SAFE_END
 	spy_cleanup(chan);
@@ -1198,6 +1193,8 @@
 	trans = (dir == SPY_READ) ? &chan->spies->read_translator : &chan->spies->write_translator;
 
 	AST_LIST_TRAVERSE(&chan->spies->list, spy, list) {
+		if (spy->status != CHANSPY_RUNNING)
+			continue;
 		ast_mutex_lock(&spy->lock);
 
 		queue = (dir == SPY_READ) ? &spy->read_queue : &spy->write_queue;
Index: apps/app_chanspy.c
===================================================================
--- apps/app_chanspy.c	(revision 69732)
+++ apps/app_chanspy.c	(working copy)
@@ -329,9 +329,10 @@
 
 	ast_deactivate_generator(chan);
 	
+	csth.spy.status = CHANSPY_DONE;
+
 	ast_mutex_lock(&csth.spy.lock);
 	if (csth.spy.chan) {
-		csth.spy.status = CHANSPY_DONE;
 		ast_mutex_lock(&csth.spy.chan->lock);
 		ast_channel_spy_remove(csth.spy.chan, &csth.spy);
 		ast_mutex_unlock(&csth.spy.chan->lock);
